#qpy:kivy
#
# A simple/portable Game Clocks for KoW.
# Copyright (C) 2017 MiKael NAVARRO
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""KoW Clocks.
Copyright (C) 2017 MiKael NAVARRO

A simple/portable Game Clocks for KoW written with Kivy.

"""

# Specific variables for pydoc
__author__ = "MiKael Navarro <klnavarro@free.fr>"
__date__ = "Sat Sep 09 2017"
__version__ = "0.1.1"
__credits__ = """Thanks to Sentdex for his tutorials."""

# Include directives
import kivy
kivy.require('1.7.2')
 
from kivy.app import App
from kivy.core.window import Window
from kivy.lang import Builder
from kivy.uix.floatlayout import FloatLayout
from kivy.properties import NumericProperty, ObjectProperty
from kivy.clock import Clock
from kivy.core.audio import SoundLoader
from kivy.logger import Logger


# Global variables
#Window.size = (640, 320)  # default resolution 16:9
#presentation = Builder.load_file("clocky.kv")
DEFAULT_GAME_PTS = 500
TIME_BY_TURN = 2.0 / 500 # 2min/500pts
MAX_TURNS = 6


#
# Main screen.
#
class Clocks(FloatLayout):
    """Main screen."""

    # Chronometers
    r_chrono = ObjectProperty()
    b_chrono = ObjectProperty()
    # Moves
    r_moves = ObjectProperty()
    b_moves = ObjectProperty()

    def countdown_red_time(self, state):
        shortbeep.play()
        if state == "down":
            self.b_chrono.state = "normal"  # reset blue button
            #self.b_chrono.size_hint = [ 1, 1 ]
            self.r_chrono.state = "down"  # start red chrono
            #self.r_chrono.size_hint = [ 1, 1.2 ]
            moves = int(self.r_moves.text.split(":")[1]) + 1  # incr red moves
            self.r_moves.text = "Move: %d" % moves

    def decr_red_moves(self):
        moves = int(self.r_moves.text.split(":")[1]) - 1  # decr red moves
        self.r_moves.text = "Move: %d" % moves
        
    def countdown_blue_time(self, state):
        shortbeep.play()
        if state == "down":
            self.r_chrono.state = "normal"  # reset red button
            #self.r_chrono.size_hint = [ 1, 1 ]
            self.b_chrono.state = "down"  # start blue chrono
            #self.b_chrono.size_hint = [ 1, 1.2 ]
            moves = int(self.b_moves.text.split(":")[1]) + 1  # incr blue moves
            self.b_moves.text = "Move: %d" % moves

    def decr_blue_moves(self):
        moves = int(self.b_moves.text.split(":")[1]) - 1  # decr blue moves
        self.b_moves.text = "Move: %d" % moves

    def update(self, dt):
        """Scheduled every 1 second to update chronometers."""

        Logger.trace("ClocksApp: dt= %f" % dt)

        if self.r_chrono.state == "down":
            minutes, seconds = self.r_chrono.text.split(":")
            chrono = int(minutes) * 60 + int(seconds) - 1
            if chrono >= 0:
                self.r_chrono.text = "%d:%02d" % (chrono / 60, chrono % 60)
            else:
                self.r_chrono.state = "normal"
                buzzer.play()  # beep!

        if self.b_chrono.state == "down":
            minutes, seconds = self.b_chrono.text.split(":")
            chrono = int(minutes) * 60 + int(seconds) - 1
            if chrono >= 0:
                self.b_chrono.text = "%d:%02d" % (chrono / 60, chrono % 60)
            else:
                self.b_chrono.state = "normal"
                buzzer.play()  # beep!
        

#
# Main App class.
#
class ClocksApp(App):
    """Judo ScoreBoard."""

    icon = "clocks.jpg"  # prior to 1.8.0

    def build_config(self, config):
        """Default configuration file 'clocks.ini'."""

        config.setdefaults("params", {
            "gamepts": DEFAULT_GAME_PTS,
            "nbturns": MAX_TURNS
        })

    def build(self):
        # Initializes the application; it will be called only once.
        root = Clocks()
        self.icon = "clocks.jpg"

        tottime = self.config.getint("params", "nbturns") \
          * self.config.getint("params", "gamepts") \
          * TIME_BY_TURN * 60
        Logger.debug("ClocksApp: config loaded (tottime= %ds)" % tottime)
        root.r_chrono.text = "%02d:%02d" % (tottime / 60, tottime % 60)
        root.b_chrono.text = "%02d:%02d" % (tottime / 60, tottime % 60)

        Clock.schedule_interval(root.update, 1)

        return root

    use_kivy_settings = False
    def build_settings(self, settings):
        """Create a settings panel."""

        jsondata = """[{ "type": "title",
                         "title": "KoW params" },
                          { "title": "Points game",
                            "desc": "Points in your game.",
                            "section": "params",
                            "key": "gamepts",
                            "type": "numeric" },
                          { "title": "Nb turns",
                            "desc": "Number of turns.",
                            "section": "params",
                            "key": "nbturns",
                            "type": "numeric" }
                      ]"""
        settings.add_json_panel("KoW clocks", self.config, data=jsondata)

    def on_config_change(self, config, section, key, value):
        """Update UI."""

        if config is self.config:
            tottime = self.config.getint("params", "nbturns") \
              * self.config.getint("params", "gamepts") \
              * TIME_BY_TURN * 60
            Logger.debug("ClocksApp: config updated (tottime= %ds)" % tottime)
            self.root.r_chrono.text = "%02d:%02d" % (tottime / 60, tottime % 60)
            self.root.b_chrono.text = "%02d:%02d" % (tottime / 60, tottime % 60)
        # Deactivate chronos
        self.root.r_chrono.state = "normal"
        self.root.b_chrono.state = "normal"
        self.root.r_moves.text = "Move: 0"
        self.root.b_moves.text = "Move: 0"

    def on_pause(self):
        # Here we can save data if needed
        return True

    def on_resume(self):
        # Here we can check if any data needs replacing (usually nothing)
        pass


#
# Main entry point:
#
if __name__ == '__main__':

    # Prepare sound
    atone = SoundLoader.load('snd/atone.ogg')
    Logger.info("ClocksApp: sound found at '%s' (%.3fs)" % (atone.source, atone.length))
    shortbeep = SoundLoader.load('snd/shortbeep.ogg')
    Logger.info("ClocksApp: sound found at '%s' (%.3fs)" % (shortbeep.source, shortbeep.length))
    buzzer = SoundLoader.load('snd/buzzer.ogg')
    Logger.info("ClocksApp: sound found at '%s' (%.3fs)" % (buzzer.source, buzzer.length))

    ClocksApp().run()

